package hr.fer.diplomski.seminar.pso;

import java.util.ArrayList;
import java.util.Random;

/**
 * Swarm represents 'population' of particles.
 * Class handles initialization of parameters and defines neighbours
 * @author marko pletikosa 0036424228
 *
 */
public class Swarm {

	private ArrayList<Particle> particles;
	private Particle bestOne;
	private Random rnd;
	private Function swarmEvalFun;

	public Swarm(int numParticle, Function fitness, double xMax, double xMin,
			double vMax, double c1, double c2) {
		particles = new ArrayList<Particle>(numParticle + 1);
		rnd = new Random();
		swarmEvalFun = fitness;
		// initialize all particles and add them to global list particles
		for (int i = 0; i < numParticle; i++) {
			Particle current;
			current = new Particle(fitness, xMax, xMin, vMax, new Random(), c1,
					c2);
			if (i == 0) {
				bestOne = current;
			}
			if (current.getBestFit() < bestOne.getBestFit()) {
				bestOne = current;
			}
			particles.add(current);

		}
		// initialize every particle's neighbours - ring like structure
		for (int i = 0; i < particles.size(); i++) {
			Particle current = particles.get(i);
			if (i == 0) {
				current.addNeighbours(particles
						.subList(i + 1, particles.size()));
				continue;
			} else if (i == (particles.size() - 1)) {
				current.addNeighbours(particles.subList(0, i));
				continue;
			}
			// if i == size -1, then sublist is from size to size = 0 neighbours
			// that is not allowed
			if (rnd.nextBoolean() == true && i != (particles.size() - 1)) {
				// your neighbours will be all the particles on the right of you
				current.addNeighbours(particles
						.subList(i + 1, particles.size()));
			} else {
				// all particles to the left of you
				current.addNeighbours(particles.subList(0, i));

			}
		}

	}

	public void iterate(int numIter, double error) {
		double globalFit;
		boolean end = false;
		for (int i = 0; i < numIter; i++) {
			for (Particle p : particles) {
				p.iteration();
				if (p.getBestFit() < bestOne.getBestFit()) {
					bestOne = p;
					globalFit = bestOne.getBestFit();
					System.out.println("Best fit changed to: "
							+ globalFit + " in iteration no. " + i);
					if (Math.abs(globalFit - swarmEvalFun.evaluate(swarmEvalFun.getOpt())) < error){
						System.out.println("In " + i + " iterations, error decreased below " + error);
						end = true;
						break;
					}
				}
			}
			if (end){
				break;
			}
		}
		double sum=0;
		for (Particle p: particles){
			sum += p.getBestFit();
		}
		System.out.println("AVG fitness: " + sum/particles.size());
	}

}
